import argparse
from collections import OrderedDict
from progress.bar import IncrementalBar

from bookstack import Bookstack
from vcinema_utils import VCinemaUtils

# Page ID of https://wiki.jacknet.io/books/vcinema/page/films-by-release-year
FILMS_BY_YEAR_PAGE_ID = 24


def build_page(films_by_year, progressbar=None):
    films_by_year_sorted = OrderedDict(sorted(films_by_year.items(), key=lambda t: t[0], reverse=True))

    page_table = "| Year | Films |\n| - | - |"

    for year in films_by_year_sorted.keys():
        page_table += "\n"
        page_table += str(year) + " | "
        page_table += "<br>".join("[{}](https://www.imdb.com/title/tt{}/)".format(film['title'], film['imdb_id']) for film in films_by_year_sorted[year])

        if progressbar is not None:
            progressbar.next()

    if progressbar is not None:
        progressbar.finish()

    return page_table


def update_films_by_year(token_id, token_secret):
    print("Retrieving VCinema viewings")
    viewings = VCinemaUtils.get_vcinema_viewings(token_id, token_secret)

    viewing_count = len(viewings)
    with IncrementalBar('Retrieving movie data', max=viewing_count, suffix='%(percent).1f%% - %(eta)ds remaining', check_tty=False) as bar:
        VCinemaUtils.add_imdb_data_to_viewings(viewings, ['year'], bar)

    with IncrementalBar('Processing viewing data', max=viewing_count, suffix='%(percent).1f%% - %(eta)ds remaining', check_tty=False) as bar:
        viewings_by_year = VCinemaUtils.filter_viewings(viewings, 'year', bar)

    year_count = len(viewings_by_year)
    with IncrementalBar('Generating table', max=year_count, suffix='%(percent).1f%% - %(eta)ds remaining', check_tty=False) as bar:
        films_by_year_page = build_page(viewings_by_year, bar)

    print("Updating page")
    Bookstack.update_page(VCinemaUtils.JACKNET_WIKI_URL, token_id, token_secret, FILMS_BY_YEAR_PAGE_ID, markdown=films_by_year_page)

    print("Done!")


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Update page displaying VCinema films sorted by release year.')
    parser.add_argument('token_id', help='API token ID.')
    parser.add_argument('token_secret', help='API token secret.')

    args = parser.parse_args()

    update_films_by_year(args.token_id, args.token_secret)
